/* Armlet68k.h
 * PalmOS specific.
 * This file contains definitions needed by both the armlet and the 68k.
 * You need to have define uint8 etc. and Call68KFuncType before including this
 */
#ifndef _ARMLET68K_H
#define _ARMLET68K_H

#include "PrefsState_PalmOS.h"

#define SNAPSHOT_VERSION_OLD 1
#define SNAPSHOT_VERSION 2

// Requesters
#define REQ_BAD_SNAPSHOT		1
#define REQ_BAD_SNAPSHOT_VERSION	2
#define REQ_BAD_OPCODE			3
#define REQ_BAD_JUMP			4
#define REQ_BAD_OPCODE_1541		5
#define REQ_BAD_JUMP_1541		6
#define REQ_NEW_DRIVE			7
#define REQ_NEW_SIDRENDERER		8
#define REQ_DEBUG			9
#define REQ_EL_TEXT			10
#define REQ_EL_GFX			11
#define REQ_DEBUGSTR			12

// Error codes
#define ERR_NEW_DISPLAY	 1
#define ERR_NEW_CPU	 2
#define ERR_NEW_JOB1541	 3
#define ERR_NEW_CPU1541	 4
#define ERR_NEW_VIC	 5
#define ERR_NEW_SID	 6
#define ERR_NEW_CIA1	 7
#define ERR_NEW_CIA2	 8
#define ERR_NEW_IEC	 9
#define ERR_NEW_FRODO	11
#define ERR_NEW_C64	12
#define ERR_NORMAL_EXIT	13

// The joystick field
#define JOYUP		1
#define JOYLEFT		2
#define JOYRIGHT	4
#define JOYDOWN		8
#define JOYFIRE		16

// Byte 2 in keyboard
#define KEYDOWN	((uint32)1)
#define KEYUP	((uint32)2)

#define CMD_DRAWFRAME		0x00000001
#define CMD_GETPREFS		0x00000002
#define CMD_SETPREFS		0x00000004
#define CMD_LOAD		0x00000008
#define CMD_SAVE		0x00000010
#define CMD_RESET		0x00000020

#define CMD68K_DRAWFRAME	0x01000000L
#define CMD68K_GETPREFS		0x02000000L
#define CMD68K_SETPREFS		0x04000000L
#define CMD68K_LOAD		0x08000000L
#define CMD68K_SAVE		0x10000000L
#define CMD68K_RESET		0x20000000L

#define FILETYPE_D64	1
#define FILETYPE_T64	2

// A minimal FILE * implementation, using chunks from a prc/pdb
#define MAX_PARTS 10
// Max size about 10*60 kB... Phone me if there's a .t64 larger than 600kB in this universe.
typedef struct {
	char name[32];		// Filename [a-zA-Z0-9 ._-]{31}
	uint32 type;		// D64/T64/???
	uint32 size;		// File length
	uint32 partSize;	// Size of one chunk
	uint32 offset;		// Read(/write) offset
	uint32 partOffset;	// Offset modulo partSize for convenience
	uint32 currentPart;
	uint32 modified;    // Flag if disk modified by emulation
	uint32 location;    // 0 -> internal; 1 -> expansion card
	struct {		// The actual parts
		uint8 *p;	// Data
		uint32 dbf;	// Database or file (68k use)
		uint32 id;	// Index in database (68k use)
		uint32 handle;	// MemHandle (68k use)
	} part[MAX_PARTS];
} FILE;

/*
typedef unsigned long Call68KFuncType(const void *emulStateP, unsigned long trapOrFunction, const void *argsOnStackP, unsigned long argsSizeAndwantA0);
*/
// All communication between 68k and arm goes through this struct.
typedef struct {
  // 68k emulation state etc
  const void *emulStateP;
  Call68KFuncType *call68k;
  uint32 r10org;
  uint32 r9org;
  uint32 r9curr;
  
  // 68k callbacks
  void  *vBlank;
  void  *requester;
  void  *malloc;
  void  *free;
	void  *dmwrite;
	
  /******** Resources ********/
	// CPU
  uint8 *ram;
  uint8 *kernalROM;
  uint8 *basicROM;
  uint8 *charROM;
  uint8 *color;
  uint8 *io_ram;
  uint8 *modetab;
  uint8 *optab;
	// 1541
  uint8 *ram1541;
  uint8 *rom1541;
  char *errors1541[12];
	// SID
  int32 *sintab;
	uint16 *triTable; uint16 *triSawTable;
	uint16 *triRectTable;
	uint16 *sawRectTable;
	uint16 *triSawRectTable;
	uint32 *egTable;
	uint8  *egdrShift;
	int16 *sampleTab;
	uint32 *resonanceLP;
	uint32 *resonanceHP;
	// VIC
	uint16 *expTable;
	uint16 *multiExpTable;
	uint32 *textColorTable;
  /******** End resources ********/

	// Preferences
	struct PrefsState *prefs;
  
  // I/O
  uint8 *displayBuf;
	uint32 displayModulo;
  int16 *soundBuf;
	uint32 joystick;	// Joystick state
	uint32 key;		// Key up/down this frame (yes, only 1 key up/down per frame!)
	uint32 pc;		// DEBUG show pc in GUI
	uint8 *snapshot1;
	uint8 *snapshot2;

  // Commands
  uint32 cmd;

  // Fatal errors
  uint32 errCode;

  FILE *disk;
	uint8 diskType;

  uint8 track1541;
  uint8 driveLED;
} ArmletData;
#endif
