#define _PALMC64COMMANDS_C

#include <PalmOS.h>

#include "PalmOSGUI.h"
#include "PalmC64Commands.h"
#include "FrodoResources.h"
#include "PalmOSFileIO.h"


char strCommandsTemp[C64COMMANDS_MAX][C64COMMANDS_MAXLEN];

//***********************************************************
// Procedure: HandleEventCommands
//
// Parameter: e   Event
//
// Returnval: True if event handled in this procedure
//
// Description: Event handler for dialog for C64 commands.
//***********************************************************
Boolean HandleEventCommands(EventPtr e)
{
	FormPtr f = FrmGetActiveForm();
	static UInt16 iLastVisibleIndex = 0xffff;
	UInt16 i;
	FieldType *pField;
	UInt16 iLen;
	
	switch (e->eType) 
	{
	  case fldEnterEvent:
      if(e->data.fldEnter.fieldID >= CommandsTitleFirst && e->data.fldEnter.fieldID <= CommandsTitle5)
      {
        i = FrmGetObjectIndex(f, CommandsCommand);
        pField = FrmGetObjectPtr(f, i);
        iLen = FldGetTextLength(pField);
        if(iLastVisibleIndex != 0xffff)
        {
          if(FldGetTextPtr(pField) != NULL)
            StrNCopy(strCommandsTemp[iLastVisibleIndex], FldGetTextPtr(pField), C64COMMANDS_MAXLEN);
        }
        FldDelete(pField, 0, iLen);
        iLastVisibleIndex = e->data.fldEnter.fieldID - CommandsTitleFirst;
        iLen = StrLen(strCommandsTemp[iLastVisibleIndex]);
        FldInsert(pField, strCommandsTemp[iLastVisibleIndex], iLen);
	    }
	    break;

    case ctlSelectEvent:
      if(iLastVisibleIndex != 0xffff && e->data.ctlSelect.controlID == CommandsOKButton)
      {
        i = FrmGetObjectIndex(f, CommandsCommand);
        pField = FrmGetObjectPtr(f, i);
        if(FldGetTextPtr(pField) != NULL)
          StrNCopy(strCommandsTemp[iLastVisibleIndex], FldGetTextPtr(pField), C64COMMANDS_MAXLEN);
      }
      break;
      
	}
	
	return false;
}


//***********************************************************
// Procedure: InitCommandsDialog
//
// Parameter: f   Form to display dialog
//
// Returnval: -
//
// Description: Edit preferences for C64 commands
//***********************************************************
void InitCommandsDialog(FormType *f)
{
  UInt16 i;

  // Set current title for each item
  for(i=0; i<C64COMMANDS_MAX; ++i)
  {
    SetText(f, CommandsTitleFirst + i, prefsPalm->C64CommandTitle[i]);
  }

  // Copy commands to temp. buffer
  MemMove(strCommandsTemp, prefsPalm->C64Command[0], C64COMMANDS_MAX * C64COMMANDS_MAXLEN);
}


//***********************************************************
// Procedure: ExitCommandsDialog
//
// Parameter: f       Form
//            choice  Flag if user selected OK button
//
// Returnval: -
//
// Description: Get settings from commands dialog
//***********************************************************
void ExitCommandsDialog(FormType *f, UInt16 choice)
{
	UInt16 i;
	UInt16 usedIndex;
  const char *strTitle;

	if (choice != CommandsOKButton) 
		return;

  for(i=0, usedIndex=0; i<C64COMMANDS_MAX; ++i)
  {
    if(StrLen(strCommandsTemp[i]) > 0)
    {
      // Command used -> copy title and command
      strTitle = GetText(f, CommandsTitleFirst + i);
      if(strTitle == 0 || StrLen(strTitle) == 0)
        // No title -> use first characters of command
        StrNCopy(prefsPalm->C64CommandTitle[usedIndex], strCommandsTemp[i], C64COMMANDS_TITLELEN);
      else
        StrNCopy(prefsPalm->C64CommandTitle[usedIndex], strTitle, C64COMMANDS_TITLELEN);
      StrNCopy(prefsPalm->C64Command[usedIndex], strCommandsTemp[i], C64COMMANDS_MAXLEN);
      ++usedIndex;
    }
  }

  for(; usedIndex < C64COMMANDS_MAX; ++usedIndex)
  {
    // Clear unused commands
    StrCopy(prefsPalm->C64CommandTitle[usedIndex], "\0");
    StrCopy(prefsPalm->C64Command[usedIndex], "\0");
  }

  SavePrefsPalm();

  // Note: Menu of MainForm reloaded after close of Commands-dialog
  // -> menuOpenEvent raised again, so new menu created there
}
