#ifndef _PALMOSGUI_H
#define _PALMOSGUI_H

typedef UInt32 uint32;
typedef Int32 int32;
typedef UInt16 uint16;
typedef Int16 int16;
typedef UInt8 uint8;
typedef Int8 int8;

#include <PceNativeCall.h>
#include "PalmOSGUI_dbfile.h"
#include "Armlet68k.h"

#define CODE2_SECTION  __attribute__ ((section ("code2sec")))

#define DEFAULT_SNAP_NAME "default snapshot"


// 4-chars-as-UInt32
#define makeType(a,b,c,d) ((((UInt32)a)<<24) | (((UInt32)b)<<16) | (((UInt32)c)<<8) | ((UInt32)d))

// Resource signatures
#define appSignature	((UInt32)	makeType('F','r','o','d'))
#define armletType	((DmResType)	makeType('A','r','m','l'))
#define romType		((DmResType)	makeType('R','O','M','z'))
#define tblType		((DmResType)	makeType('T','a','b','l'))
#define strType		((DmResType)	makeType('S','t','r','g'))
#define prefsType	((DmResType)	makeType('P','r','e','f'))
#define bitmapType	((DmResType)	makeType('T','b','m','p'))
#define dataType	((DmResType)	makeType('D','A','T','A'))
#define mainType	((DmResType)	makeType('M','A','I','N'))
#define crashType	((DmResType)	makeType('C','R','S','H'))
#define dbDataType	((UInt32)	makeType('d','a','t','a'))
#define stringType  ((UInt32)	makeType('t','S','T','R'))

// Resource indexes. Why 32? Well, i like some space...
// It is CRITICAL that these indexes agree with precalc.c and the Makefile!

// ROMz
#define IDX_KERNAL	32
#define IDX_BASIC	33
#define IDX_CHAR	34
#define IDX_1541	35

// Tabl
#define IDX_EXPTABLE		32
#define IDX_MULTIEXPTABLE	33
#define IDX_TEXTCOLORTABLE	34
#define IDX_TRITABLE		35
#define IDX_TRISAWTABLE		36
#define IDX_TRIRECTTABLE	37
#define IDX_SAWRECTTABLE	38
#define IDX_TRISAWRECTTABLE	39
#define IDX_EGTABLE		40
#define IDX_EGDRSHIFT		41
#define IDX_SAMPLETAB		42
#define IDX_SINTAB		43
#define IDX_RESONANCELP		44
#define IDX_RESONANCEHP		45
#define IDX_MODETAB 46
#define IDX_OPTAB 47

// Strg
#define IDX_ERRORS1541		32

// Arml
#define IDX_ARMLET		32

// Pref
#define IDX_PREFS		32
#define IDX_PALMPREFS 33

// CRSH
#define IDX_CRASH		34


// 0x80d0 0x80e0
// 0x8c54 0x8415
#define SNAPSHOT_SIZE_1 0x9200L
#define SNAPSHOT_SIZE_2 0x8f00L

// Sound
#define SOUND_BUFSIZE 22050L
#define FRAME_SAMPLES 882


// Declarations
void CheckCrash(void) CODE2_SECTION;
void NoCrash(void) CODE2_SECTION;
UInt32 Launch(void);
UInt32 RunArmlet(void);
void SetPaused(Boolean b);
MemHandle GetResource(DmResType type, DmResID resID);
void FreeResource(MemHandle resHandle);

// Keyboard and joystick
void SetKeyMask(void);
char *StringForKey(UInt32 k);
void CheckJoyPen(void);
UInt32 CheckJoyPen2(int32 x, int32 y, int32 szx, int32 szy);
void CheckJoyKey(void);
void CheckKeyKey(void);

// Graphics
UInt32 InitGraphicsSound(void);
void FreeGraphicsSound(void);
UInt32 ChangeScreenMode(UInt32 sm, Boolean first, Boolean defaults);
void RepaintStatistics(UInt32 percentage);
void RepaintTrackNum(UInt32 track, UInt8 led);
void RepaintOnScreenKeyboard(FormType *f);
void UpdateOnScreenKeyboard(FormType *f);
void UpdateOnScreenJoystick(void);
void RepaintSCIndicator(Boolean SCOn);
void Repaint1541Indicator(Boolean On1541);

// Sound
UInt32 InitSound(void);
void FreeSound(void);
void StartSound(void);
void StopSound(void);
void PauseSound(Boolean b);
Err SoundCallback(void *userDataP, SndStreamRef stream, void *bufferP, UInt32 size);
void MakeNoise(Boolean b, UInt32 v);
void SoundCurrFrame(void);

// Dialogs and event handler
void ColorsTableDraw(void *pTable, Int16 row, Int16 col, RectangleType *bounds);
void KeyboardTableDrawItem(UInt32 lIndex);
void KeyboardTableDraw(void *pTable, Int16 row, Int16 col, RectangleType *bounds);
Boolean SetValue(FormType *f, UInt16 id, UInt16 v);
UInt16 GetValue(FormType *f, UInt16 id);
Boolean SetText(FormType *f, UInt16 id, char *s);
const char *GetText(FormType *f, UInt16 id);
void InitColorsDialog(FormType *f) CODE2_SECTION;
void InitKeyboardDialog(FormType *f) CODE2_SECTION;
void InitGfxSoundDialog(FormType *f) CODE2_SECTION;
void InitJoystickDialog(FormType *f) CODE2_SECTION;
void InitObscureDialog(FormType *f) CODE2_SECTION;
void InitSampleKeyDialog(FormType *f) CODE2_SECTION;
void ExitColorsDialog(FormType *f, UInt16 choice) CODE2_SECTION;
void ExitGfxSoundDialog(FormType *f, UInt16 choice) CODE2_SECTION;
void ExitObscureDialog(FormType *f, UInt16 choice) CODE2_SECTION;
char *ShowSaveAsDialog(char *s);
UInt32 SampleKey(UInt32 org);


// Arm->68k callbacks
UInt32 VBlank68k(void *);
Int32 ShowRequester68k(Int32 *);
void *Malloc68k(UInt32);
UInt32 Free68k(void *);
Int32 DmWrite68k(Int32 *);


#ifndef _PALMOSGUI_C
// Global variables from main part
extern Boolean exiting;
extern Boolean crashed;
extern Boolean firstTime;
extern struct PrefsState *prefs;
extern struct PreferencesPalm *prefsPalm;
extern struct PrefsState prefsData;
extern struct PreferencesPalm prefsPalmData;
extern UInt8 canSound, can256, is320;
extern UInt32 screenW, screenH;
extern RGBColorType colorTablePreset[16*6];
extern Boolean doFileLoadAfterSnap;
extern Boolean bLaunchedFromSD;
#endif

#endif
